@@ -120,14 +120,10 @@ WERROR NetGroupAdd_r(struct libnetapi_ctx *ctx,
 					       &rid,
 					       &result);
 
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	switch (r->in.level) {
 		case 1:
@@ -152,14 +148,11 @@ WERROR NetGroupAdd_r(struct libnetapi_ctx *ctx,
 								  GROUPINFODESCRIPTION,
 								  &info,
 								  &result);
-				if (!NT_STATUS_IS_OK(status)) {
+				if (any_nt_status_not_ok(
+					    status, result, &status)) {
 					werr = ntstatus_to_werror(status);
 					goto failed;
 				}
-				if (!NT_STATUS_IS_OK(result)) {
-					werr = ntstatus_to_werror(result);
-					goto failed;
-				}
 			}
 
 			if (info2->grpi2_attributes != 0) {
@@ -182,14 +175,11 @@ WERROR NetGroupAdd_r(struct libnetapi_ctx *ctx,
 								  GROUPINFODESCRIPTION,
 								  &info,
 								  &result);
-				if (!NT_STATUS_IS_OK(status)) {
+				if (any_nt_status_not_ok(
+					    status, result, &status)) {
 					werr = ntstatus_to_werror(status);
 					goto failed;
 				}
-				if (!NT_STATUS_IS_OK(result)) {
-					werr = ntstatus_to_werror(result);
-					goto failed;
-				}
 			}
 
 			if (info3->grpi3_attributes != 0) {
@@ -205,14 +195,10 @@ WERROR NetGroupAdd_r(struct libnetapi_ctx *ctx,
 			break;
 	}
 
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto failed;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto failed;
-	}
 
 	werr = WERR_OK;
 	goto done;
@@ -300,14 +286,10 @@ WERROR NetGroupDel_r(struct libnetapi_ctx *ctx,
 					 &rids,
 					 &types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	if (rids.count != 1) {
 		werr = WERR_BAD_NET_RESP;
@@ -333,28 +315,20 @@ WERROR NetGroupDel_r(struct libnetapi_ctx *ctx,
 				       rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	status = dcerpc_samr_QueryGroupInfo(b, talloc_tos(),
 					    &group_handle,
 					    GROUPINFOATTRIBUTES,
 					    &info,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 #if 0
 	/* breaks against NT4 */
@@ -367,14 +341,10 @@ WERROR NetGroupDel_r(struct libnetapi_ctx *ctx,
 					      &group_handle,
 					      &rid_array,
 					      &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	{
 	struct lsa_Strings names;
@@ -387,14 +357,10 @@ WERROR NetGroupDel_r(struct libnetapi_ctx *ctx,
 					&names,
 					&member_types,
 					&result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (names.count != rid_array->count) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -411,27 +377,19 @@ WERROR NetGroupDel_r(struct libnetapi_ctx *ctx,
 						       &group_handle,
 						       rid_array->rids[i],
 						       &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	status = dcerpc_samr_DeleteDomainGroup(b, talloc_tos(),
 					       &group_handle,
 					       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	ZERO_STRUCT(group_handle);
 
@@ -520,14 +478,10 @@ WERROR NetGroupSetInfo_r(struct libnetapi_ctx *ctx,
 					 &rids,
 					 &types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (rids.count != 1) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -549,14 +503,10 @@ WERROR NetGroupSetInfo_r(struct libnetapi_ctx *ctx,
 				       rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	switch (r->in.level) {
 		case 0:
@@ -585,14 +535,10 @@ WERROR NetGroupSetInfo_r(struct libnetapi_ctx *ctx,
 							  GROUPINFODESCRIPTION,
 							  &info,
 							  &result);
-			if (!NT_STATUS_IS_OK(status)) {
+			if (any_nt_status_not_ok(status, result, &status)) {
 				werr = ntstatus_to_werror(status);
 				goto done;
 			}
-			if (!NT_STATUS_IS_OK(result)) {
-				werr = ntstatus_to_werror(result);
-				goto done;
-			}
 
 			info.attributes.attributes = g2->grpi2_attributes;
 			status = dcerpc_samr_SetGroupInfo(b, talloc_tos(),
@@ -609,14 +555,10 @@ WERROR NetGroupSetInfo_r(struct libnetapi_ctx *ctx,
 							  GROUPINFODESCRIPTION,
 							  &info,
 							  &result);
-			if (!NT_STATUS_IS_OK(status)) {
+			if (any_nt_status_not_ok(status, result, &status)) {
 				werr = ntstatus_to_werror(status);
 				goto done;
 			}
-			if (!NT_STATUS_IS_OK(result)) {
-				werr = ntstatus_to_werror(result);
-				goto done;
-			}
 
 			info.attributes.attributes = g3->grpi3_attributes;
 			status = dcerpc_samr_SetGroupInfo(b, talloc_tos(),
@@ -648,14 +590,10 @@ WERROR NetGroupSetInfo_r(struct libnetapi_ctx *ctx,
 			break;
 	}
 
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	werr = WERR_OK;
 
@@ -798,14 +736,10 @@ WERROR NetGroupGetInfo_r(struct libnetapi_ctx *ctx,
 					 &rids,
 					 &types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (rids.count != 1) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -826,14 +760,10 @@ WERROR NetGroupGetInfo_r(struct libnetapi_ctx *ctx,
 				       rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	status = dcerpc_samr_QueryGroupInfo(b, talloc_tos(),
 					    &group_handle,
@@ -974,14 +904,10 @@ WERROR NetGroupAddUser_r(struct libnetapi_ctx *ctx,
 				       rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	init_lsa_String(&lsa_user_name, r->in.user_name);
 
@@ -1019,14 +945,10 @@ WERROR NetGroupAddUser_r(struct libnetapi_ctx *ctx,
 					    rids.ids[0],
 					    7, /* why ? */
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	werr = WERR_OK;
 
@@ -1134,14 +1056,10 @@ WERROR NetGroupDelUser_r(struct libnetapi_ctx *ctx,
 				       rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	init_lsa_String(&lsa_user_name, r->in.user_name);
 
@@ -1179,14 +1097,10 @@ WERROR NetGroupDelUser_r(struct libnetapi_ctx *ctx,
 					       &group_handle,
 					       rids.ids[0],
 					       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	werr = WERR_OK;
 
@@ -1423,14 +1337,10 @@ WERROR NetGroupEnum_r(struct libnetapi_ctx *ctx,
 					     2,
 					     &domain_info,
 					     &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	if (r->out.total_entries) {
 		*r->out.total_entries = domain_info->general.num_groups;
@@ -1572,14 +1482,10 @@ WERROR NetGroupGetUsers_r(struct libnetapi_ctx *ctx,
 					 &group_rids,
 					 &name_types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (group_rids.count != 1) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -1595,27 +1501,19 @@ WERROR NetGroupGetUsers_r(struct libnetapi_ctx *ctx,
 				       group_rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	status = dcerpc_samr_QueryGroupMember(b, talloc_tos(),
 					      &group_handle,
 					      &rid_array,
 					      &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	status = dcerpc_samr_LookupRids(b, talloc_tos(),
 					&domain_handle,
@@ -1624,14 +1522,10 @@ WERROR NetGroupGetUsers_r(struct libnetapi_ctx *ctx,
 					&names,
 					&member_types,
 					&result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (names.count != rid_array->count) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -1764,14 +1658,10 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 					 &group_rids,
 					 &group_types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 	if (group_rids.count != 1) {
 		werr = WERR_BAD_NET_RESP;
 		goto done;
@@ -1790,28 +1680,20 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 				       group_rids.ids[0],
 				       &group_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	status = dcerpc_samr_QueryGroupInfo(b, talloc_tos(),
 					    &group_handle,
 					    GROUPINFOATTRIBUTES,
 					    &group_info,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	switch (r->in.level) {
 		case 0:
@@ -1849,14 +1731,10 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 					 &user_rids,
 					 &name_types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	if (r->in.num_entries != user_rids.count) {
 		werr = WERR_BAD_NET_RESP;
@@ -1873,14 +1751,10 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 					      &group_handle,
 					      &rid_array,
 					      &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	/* add list */
 
@@ -1930,14 +1804,10 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 						    add_rids[i],
 						    7 /* ? */,
 						    &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	/* del list */
@@ -1947,14 +1817,10 @@ WERROR NetGroupSetUsers_r(struct libnetapi_ctx *ctx,
 						       &group_handle,
 						       del_rids[i],
 						       &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	werr = WERR_OK;
