@@ -22,18 +22,21 @@
 #define __UTIL_REPARSE_H__
 
 NTSTATUS fsctl_get_reparse_point(struct files_struct *fsp,
-				 TALLOC_CTX *ctx,
-				 char **out_data,
+				 TALLOC_CTX *mem_ctx,
+				 uint32_t *_reparse_tag,
+				 uint8_t **_out_data,
 				 uint32_t max_out_len,
-				 uint32_t *out_len);
+				 uint32_t *_out_len);
+NTSTATUS fsctl_get_reparse_tag(struct files_struct *fsp,
+			       uint32_t *_reparse_tag);
 
 NTSTATUS fsctl_set_reparse_point(struct files_struct *fsp,
-				 TALLOC_CTX *ctx,
+				 TALLOC_CTX *mem_ctx,
 				 const uint8_t *in_data,
 				 uint32_t in_len);
 
 NTSTATUS fsctl_del_reparse_point(struct files_struct *fsp,
-				 TALLOC_CTX *ctx,
+				 TALLOC_CTX *mem_ctx,
 				 const uint8_t *in_data,
 				 uint32_t in_len);
 
