@@ -32,8 +32,9 @@
 #include "transfer_file.h"
 #include "auth.h"
 #include "messages.h"
-#include "../librpc/gen_ndr/open_files.h"
+#include "librpc/gen_ndr/ndr_open_files.h"
 #include "lib/util/tevent_ntstatus.h"
+#include "source3/smbd/dir.h"
 
 /****************************************************************************
  Run a file if it is a magic script.
@@ -149,17 +150,21 @@ static NTSTATUS check_magic(struct files_struct *fsp)
  Delete all streams
 ****************************************************************************/
 
-NTSTATUS delete_all_streams(connection_struct *conn,
-			const struct smb_filename *smb_fname)
+NTSTATUS delete_all_streams(struct files_struct *fsp,
+			    struct files_struct *dirfsp,
+			    struct smb_filename *fsp_atname)
 {
+	struct smb_filename *smb_fname = fsp->fsp_name;
 	struct stream_struct *stream_info = NULL;
 	unsigned int i;
 	unsigned int num_streams = 0;
 	TALLOC_CTX *frame = talloc_stackframe();
 	NTSTATUS status;
 
-	status = vfs_fstreaminfo(smb_fname->fsp, talloc_tos(),
-				&num_streams, &stream_info);
+	status = vfs_fstreaminfo(fsp,
+				 talloc_tos(),
+				 &num_streams,
+				 &stream_info);
 
 	if (NT_STATUS_EQUAL(status, NT_STATUS_NOT_IMPLEMENTED)) {
 		DEBUG(10, ("no streams around\n"));
@@ -189,25 +194,23 @@ NTSTATUS delete_all_streams(connection_struct *conn,
 			continue;
 		}
 
-		status = synthetic_pathref(talloc_tos(),
-					   conn->cwd_fsp,
-					   smb_fname->base_name,
-					   stream_info[i].name,
-					   NULL,
-					   smb_fname->twrp,
-					   (smb_fname->flags &
-					    ~SMB_FILENAME_POSIX_PATH),
-					   &smb_fname_stream);
+		smb_fname_stream = synthetic_smb_fname(
+			talloc_tos(),
+			fsp_atname->base_name,
+			stream_info[i].name,
+			NULL,
+			smb_fname->twrp,
+			(smb_fname->flags & ~SMB_FILENAME_POSIX_PATH));
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0, ("talloc_aprintf failed\n"));
 			status = NT_STATUS_NO_MEMORY;
 			goto fail;
 		}
 
-		res = SMB_VFS_UNLINKAT(conn,
-				conn->cwd_fsp,
-				smb_fname_stream,
-				0);
+		res = SMB_VFS_UNLINKAT(dirfsp->conn,
+				       dirfsp,
+				       smb_fname_stream,
+				       0);
 
 		if (res == -1) {
 			status = map_nt_error_from_unix(errno);
@@ -238,19 +241,20 @@ static bool has_other_nonposix_opens_fn(
 	struct has_other_nonposix_opens_state *state = private_data;
 	struct files_struct *fsp = state->fsp;
 
-	if (e->name_hash != fsp->name_hash) {
+	if (e->flags & SHARE_MODE_FLAG_POSIX_OPEN) {
 		return false;
 	}
-	if ((fsp->posix_flags & FSP_POSIX_FLAGS_OPEN) &&
-	    (e->flags & SHARE_MODE_FLAG_POSIX_OPEN)) {
-		return false;
-	}
-	if (e->share_file_id == fh_get_gen_id(fsp->fh)) {
-		struct server_id self = messaging_server_id(
-			fsp->conn->sconn->msg_ctx);
-		if (server_id_equal(&self, &e->pid)) {
+	if (fsp != NULL) {
+		if (e->name_hash != fsp->name_hash) {
 			return false;
 		}
+		if (e->share_file_id == fh_get_gen_id(fsp->fh)) {
+			struct server_id self = messaging_server_id(
+				fsp->conn->sconn->msg_ctx);
+			if (server_id_equal(&self, &e->pid)) {
+				return false;
+			}
+		}
 	}
 	if (share_entry_stale_pid(e)) {
 		return false;
@@ -274,6 +278,60 @@ bool has_other_nonposix_opens(struct share_mode_lock *lck,
 	return state.found_another;
 }
 
+struct has_delete_access_opens_state {
+	bool delete_access;
+	bool delete_pending;
+};
+
+static bool has_delete_access_opens_fn(
+	struct share_mode_entry *e,
+	bool *modified,
+	void *private_data)
+{
+	struct has_delete_access_opens_state *state = private_data;
+
+	if (share_entry_stale_pid(e)) {
+		return false;
+	}
+
+	if (e->access_mask & SEC_STD_DELETE) {
+		state->delete_access = true;
+	}
+	return false;
+}
+
+bool has_delete_opens(struct files_struct *fsp,
+		      struct share_mode_lock *lck,
+		      bool *delete_access,
+		      bool *delete_pending)
+{
+	struct has_delete_access_opens_state state = {};
+	bool ok;
+
+	ok = share_mode_forall_entries(
+		lck, has_delete_access_opens_fn, &state);
+	if (!ok) {
+		return false;
+	}
+
+	*delete_access = state.delete_access;
+	*delete_pending = is_delete_on_close_set(lck, fsp->name_hash);
+	return true;
+}
+
+bool has_nonposix_opens(struct share_mode_lock *lck)
+{
+	struct has_other_nonposix_opens_state state = {};
+	bool ok;
+
+	ok = share_mode_forall_entries(
+		lck, has_other_nonposix_opens_fn, &state);
+	if (!ok) {
+		return false;
+	}
+	return state.found_another;
+}
+
 struct close_share_mode_lock_state {
 	struct share_mode_entry_prepare_state prepare_state;
 	const char *object_type;
@@ -281,6 +339,7 @@ struct close_share_mode_lock_state {
 	enum file_close_type close_type;
 	bool delete_object;
 	bool got_tokens;
+	struct smb2_lease_key parent_lease_key;
 	const struct security_unix_token *del_token;
 	const struct security_token *del_nt_token;
 	bool reset_delete_on_close;
@@ -303,6 +362,17 @@ static void close_share_mode_lock_prepare(struct share_mode_lock *lck,
 	 */
 	*keep_locked = false;
 
+	if (fsp->current_lock_count > 0) {
+		/*
+		 * Remove the byte-range locks under the glock
+		 */
+		*keep_locked = true;
+	}
+
+	if (fh_get_refcount(fsp->fh) > 1) {
+		return;
+	}
+
 	if (fsp->oplock_type != NO_OPLOCK) {
 		ok = remove_share_oplock(lck, fsp);
 		if (!ok) {
@@ -382,8 +452,11 @@ static void close_share_mode_lock_prepare(struct share_mode_lock *lck,
 	 */
 	*keep_locked = true;
 
-	state->got_tokens = get_delete_on_close_token(lck, fsp->name_hash,
-					&state->del_nt_token, &state->del_token);
+	state->got_tokens = get_delete_on_close_token(lck,
+						      fsp->name_hash,
+						      &state->del_nt_token,
+						      &state->del_token,
+						      &state->parent_lease_key);
 	if (state->close_type != ERROR_CLOSE) {
 		SMB_ASSERT(state->got_tokens);
 	}
@@ -453,6 +526,12 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 		return status;
 	}
 
+	locking_close_file(fsp, close_type);
+
+	if (fh_get_refcount(fsp->fh) > 1) {
+		goto done;
+	}
+
 	/* Remove the oplock before potentially deleting the file. */
 	if (fsp->oplock_type != NO_OPLOCK) {
 		release_file_oplock(fsp);
@@ -473,8 +552,8 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 	 */
 	lck_state.cleanup_fn = close_share_mode_lock_cleanup;
 
-	DEBUG(5,("close_remove_share_mode: file %s. Delete on close was set "
-		 "- deleting file.\n", fsp_str_dbg(fsp)));
+	DBG_INFO("%s. Delete on close was set - deleting file.\n",
+		 fsp_str_dbg(fsp));
 
 	/*
 	 * Don't try to update the write time when we delete the file
@@ -486,10 +565,9 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 	{
 		/* Become the user who requested the delete. */
 
-		DEBUG(5,("close_remove_share_mode: file %s. "
-			"Change user to uid %u\n",
-			fsp_str_dbg(fsp),
-			(unsigned int)lck_state.del_token->uid));
+		DBG_INFO("file %s. Change user to uid %u\n",
+			 fsp_str_dbg(fsp),
+			 (unsigned int)lck_state.del_token->uid);
 
 		if (!push_sec_ctx()) {
 			smb_panic("close_remove_share_mode: file %s. failed to push "
@@ -510,9 +588,10 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 
 	tmp_status = vfs_stat_fsp(fsp);
 	if (!NT_STATUS_IS_OK(tmp_status)) {
-		DEBUG(5,("close_remove_share_mode: file %s. Delete on close "
+		DBG_INFO("file %s. Delete on close "
 			 "was set and stat failed with error %s\n",
-			 fsp_str_dbg(fsp), nt_errstr(tmp_status)));
+			 fsp_str_dbg(fsp),
+			 nt_errstr(tmp_status));
 		/*
 		 * Don't save the errno here, we ignore this error
 		 */
@@ -523,24 +602,34 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 
 	if (!file_id_equal(&fsp->file_id, &id)) {
 		struct file_id_buf ftmp1, ftmp2;
-		DEBUG(5,("close_remove_share_mode: file %s. Delete on close "
+		DBG_INFO("file %s. Delete on close "
 			 "was set and dev and/or inode does not match\n",
-			 fsp_str_dbg(fsp)));
-		DEBUG(5,("close_remove_share_mode: file %s. stored file_id %s, "
-			 "stat file_id %s\n",
+			 fsp_str_dbg(fsp));
+		DBG_INFO("file %s. stored file_id %s, stat file_id %s\n",
 			 fsp_str_dbg(fsp),
 			 file_id_str_buf(fsp->file_id, &ftmp1),
-			 file_id_str_buf(id, &ftmp2)));
+			 file_id_str_buf(id, &ftmp2));
 		/*
 		 * Don't save the errno here, we ignore this error
 		 */
 		goto done;
 	}
 
+	status = parent_pathref(talloc_tos(),
+				conn->cwd_fsp,
+				fsp->fsp_name,
+				&parent_fname,
+				&base_fname);
+	if (!NT_STATUS_IS_OK(status)) {
+		goto done;
+	}
+
 	if ((conn->fs_capabilities & FILE_NAMED_STREAMS)
 	    && !fsp_is_alternate_stream(fsp)) {
 
-		status = delete_all_streams(conn, fsp->fsp_name);
+		status = delete_all_streams(fsp,
+					    parent_fname->fsp,
+					    base_fname);
 
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(5, ("delete_all_streams failed: %s\n",
@@ -564,15 +653,6 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 		fsp->fsp_flags.kernel_share_modes_taken = false;
 	}
 
-	status = parent_pathref(talloc_tos(),
-				conn->cwd_fsp,
-				fsp->fsp_name,
-				&parent_fname,
-				&base_fname);
-	if (!NT_STATUS_IS_OK(status)) {
-		goto done;
-	}
-
 	ret = SMB_VFS_UNLINKAT(conn,
 			       parent_fname->fsp,
 			       base_fname,
@@ -588,9 +668,10 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 		 * zero.
 		 */
 
-		DEBUG(5,("close_remove_share_mode: file %s. Delete on close "
+		DBG_INFO("file %s. Delete on close "
 			 "was set and unlink failed with error %s\n",
-			 fsp_str_dbg(fsp), strerror(errno)));
+			 fsp_str_dbg(fsp),
+			 strerror(errno));
 
 		status = map_nt_error_from_unix(errno);
 	}
@@ -603,6 +684,7 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
  	 */
 
 	fsp->fsp_flags.delete_on_close = false;
+	fsp->fsp_flags.fstat_before_close = false;
 	lck_state.reset_delete_on_close = true;
 
  done:
@@ -631,21 +713,28 @@ static NTSTATUS close_remove_share_mode(files_struct *fsp,
 		smb_panic("share_mode_entry_prepare_unlock() failed!");
 	}
 
-	if (lck_state.delete_object) {
-		/*
-		 * Do the notification after we released the share
-		 * mode lock. Inside notify_fname we take out another
-		 * tdb lock. With ctdb also accessing our databases,
-		 * this can lead to deadlocks. Putting this notify
-		 * after the TALLOC_FREE(lck) above we avoid locking
-		 * two records simultaneously. Notifies are async and
-		 * informational only, so calling the notify_fname
-		 * without holding the share mode lock should not do
-		 * any harm.
-		 */
-		notify_fname(conn, NOTIFY_ACTION_REMOVED,
+	if (lck_state.delete_object && NT_STATUS_IS_OK(status)) {
+		const struct smb2_lease *lease = fsp_get_smb2_lease(fsp);
+
+		if (lease != NULL) {
+			/*
+			 * If parent lease key of handle on which delete
+			 * disposition was set does not match the parent key of
+			 * last closed handle, break all leases on the parent
+			 * directory.
+			 */
+			if (!smb2_lease_key_equal(&lease->parent_lease_key,
+						  &lck_state.parent_lease_key))
+			{
+				lease = NULL;
+			}
+		}
+		notify_fname(conn,
+			     NOTIFY_ACTION_REMOVED |
+			     NOTIFY_ACTION_DIRLEASE_BREAK,
 			     FILE_NOTIFY_CHANGE_FILE_NAME,
-			     fsp->fsp_name->base_name);
+			     fsp->fsp_name,
+			     lease);
 	}
 
 	return status;
@@ -762,7 +851,7 @@ static void assert_no_pending_aio(struct files_struct *fsp,
 		/*
 		 * fsp->aio_requests and the contents (fsp->aio_requests[x])
 		 * are both independently owned by fsp and are not in a
-		 * talloc heirarchy. This allows the fsp->aio_requests array to
+		 * talloc hierarchy. This allows the fsp->aio_requests array to
 		 * be reallocated independently of the array contents so it can
 		 * grow on demand.
 		 *
@@ -896,17 +985,20 @@ static NTSTATUS close_normal_file(struct smb_request *req, files_struct *fsp,
 		fsp->op->global->durable = false;
 	}
 
+	if (fsp->fsp_flags.modified) {
+		notify_fname(conn,
+			     NOTIFY_ACTION_DIRLEASE_BREAK,
+			     0,
+			     fsp->fsp_name,
+			     fsp_get_smb2_lease(fsp));
+	}
+
 	/* If this is an old DOS or FCB open and we have multiple opens on
 	   the same handle we only have one share mode. Ensure we only remove
 	   the share mode on the last close. */
 
-	if (fh_get_refcount(fsp->fh) == 1) {
-		/* Should we return on error here... ? */
-		tmp = close_remove_share_mode(fsp, close_type);
-		status = ntstatus_keeperror(status, tmp);
-	}
-
-	locking_close_file(fsp, close_type);
+	tmp = close_remove_share_mode(fsp, close_type);
+	status = ntstatus_keeperror(status, tmp);
 
 	/*
 	 * Ensure pending modtime is set before closing underlying fd.
@@ -938,42 +1030,26 @@ static NTSTATUS close_normal_file(struct smb_request *req, files_struct *fsp,
 
 	return status;
 }
-/****************************************************************************
- Function used by reply_rmdir to delete an entire directory
- tree recursively. Return True on ok, False on fail.
-****************************************************************************/
 
-NTSTATUS recursive_rmdir(TALLOC_CTX *ctx,
-		     connection_struct *conn,
-		     struct smb_filename *smb_dname)
+static NTSTATUS recursive_rmdir_fsp(struct files_struct *fsp)
 {
+	struct connection_struct *conn = fsp->conn;
 	const char *dname = NULL;
 	char *talloced = NULL;
-	long offset = 0;
 	struct smb_Dir *dir_hnd = NULL;
 	struct files_struct *dirfsp = NULL;
 	int retval;
 	NTSTATUS status = NT_STATUS_OK;
 
-	SMB_ASSERT(!is_ntfs_stream_smb_fname(smb_dname));
-
-	status = OpenDir(talloc_tos(),
-			 conn,
-			 smb_dname,
-			 NULL,
-			 0,
-			 &dir_hnd);
+	status = OpenDir_from_pathref(talloc_tos(), fsp, NULL, 0, &dir_hnd);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
 
 	dirfsp = dir_hnd_fetch_fsp(dir_hnd);
 
-	while ((dname = ReadDirName(dir_hnd, &offset, NULL, &talloced))) {
+	while ((dname = ReadDirName(dir_hnd, &talloced))) {
 		struct smb_filename *atname = NULL;
-		struct smb_filename *smb_dname_full = NULL;
-		char *fullname = NULL;
-		bool do_break = true;
 		int unlink_flags = 0;
 
 		if (ISDOT(dname) || ISDOTDOT(dname)) {
@@ -981,56 +1057,56 @@ NTSTATUS recursive_rmdir(TALLOC_CTX *ctx,
 			continue;
 		}
 
-		/* Construct the full name. */
-		fullname = talloc_asprintf(ctx,
-				"%s/%s",
-				smb_dname->base_name,
-				dname);
-		if (!fullname) {
+		atname = synthetic_smb_fname(talloc_tos(),
+					     dname,
+					     NULL,
+					     NULL,
+					     dirfsp->fsp_name->twrp,
+					     dirfsp->fsp_name->flags);
+		TALLOC_FREE(talloced);
+		dname = NULL;
+
+		if (atname == NULL) {
 			status = NT_STATUS_NO_MEMORY;
-			goto err_break;
+			break;
 		}
 
-		smb_dname_full = synthetic_smb_fname(talloc_tos(),
-						fullname,
-						NULL,
-						NULL,
-						smb_dname->twrp,
-						smb_dname->flags);
-		if (smb_dname_full == NULL) {
-			status = NT_STATUS_NO_MEMORY;
-			goto err_break;
+		{
+			struct name_compare_entry *veto_list = conn->veto_list;
+
+			/*
+			 * Sneaky hack to be able to open veto files
+			 * with openat_pathref_fsp
+			 */
+
+			conn->veto_list = NULL;
+			status = openat_pathref_fsp_lcomp(
+				dirfsp,
+				atname,
+				UCF_POSIX_PATHNAMES /* no ci fallback */);
+			conn->veto_list = veto_list;
 		}
 
-		if (SMB_VFS_LSTAT(conn, smb_dname_full) != 0) {
-			status = map_nt_error_from_unix(errno);
-			goto err_break;
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(atname);
+			if (NT_STATUS_EQUAL(status,
+					    NT_STATUS_OBJECT_NAME_NOT_FOUND)) {
+				/* race between readdir and unlink */
+				continue;
+			}
+			break;
 		}
 
-		if (smb_dname_full->st.st_ex_mode & S_IFDIR) {
-			status = recursive_rmdir(ctx, conn, smb_dname_full);
+		if (atname->st.st_ex_mode & S_IFDIR) {
+			status = recursive_rmdir_fsp(atname->fsp);
 			if (!NT_STATUS_IS_OK(status)) {
-				goto err_break;
+				TALLOC_FREE(atname);
+				break;
 			}
 			unlink_flags = AT_REMOVEDIR;
 		}
 
-		status = synthetic_pathref(talloc_tos(),
-					   dirfsp,
-					   dname,
-					   NULL,
-					   &smb_dname_full->st,
-					   smb_dname_full->twrp,
-					   smb_dname_full->flags,
-					   &atname);
-		if (!NT_STATUS_IS_OK(status)) {
-			goto err_break;
-		}
-
 		if (!is_visible_fsp(atname->fsp)) {
-			TALLOC_FREE(smb_dname_full);
-			TALLOC_FREE(fullname);
-			TALLOC_FREE(talloced);
 			TALLOC_FREE(atname);
 			continue;
 		}
@@ -1041,55 +1117,52 @@ NTSTATUS recursive_rmdir(TALLOC_CTX *ctx,
 					  unlink_flags);
 		if (retval != 0) {
 			status = map_nt_error_from_unix(errno);
-			goto err_break;
-		}
-
-		/* Successful iteration. */
-		do_break = false;
-
-	 err_break:
-		TALLOC_FREE(smb_dname_full);
-		TALLOC_FREE(fullname);
-		TALLOC_FREE(talloced);
-		TALLOC_FREE(atname);
-		if (do_break) {
+			TALLOC_FREE(atname);
 			break;
 		}
+
+		TALLOC_FREE(atname);
 	}
+
 	TALLOC_FREE(dir_hnd);
 	return status;
 }
 
+NTSTATUS recursive_rmdir(TALLOC_CTX *ctx,
+			 connection_struct *conn,
+			 struct smb_filename *smb_dname)
+{
+	NTSTATUS status;
+
+	SMB_ASSERT(!is_ntfs_stream_smb_fname(smb_dname));
+
+	status = openat_pathref_fsp(conn->cwd_fsp, smb_dname);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
+	status = recursive_rmdir_fsp(smb_dname->fsp);
+	return status;
+}
+
 /****************************************************************************
  The internals of the rmdir code - called elsewhere.
 ****************************************************************************/
 
-static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
+static NTSTATUS rmdir_internals(struct files_struct *fsp,
+				struct files_struct *dirfsp,
+				struct smb_filename *at_fname)
 {
 	struct connection_struct *conn = fsp->conn;
 	struct smb_filename *smb_dname = fsp->fsp_name;
-	struct smb_filename *parent_fname = NULL;
-	struct smb_filename *at_fname = NULL;
-	const char *dname = NULL;
-	char *talloced = NULL;
-	long dirpos = 0;
 	struct smb_Dir *dir_hnd = NULL;
-	struct files_struct *dirfsp = NULL;
+	struct stat_ex st = {};
 	int unlink_flags = 0;
 	NTSTATUS status;
 	int ret;
 
 	SMB_ASSERT(!is_ntfs_stream_smb_fname(smb_dname));
 
-	status = parent_pathref(talloc_tos(),
-				conn->cwd_fsp,
-				fsp->fsp_name,
-				&parent_fname,
-				&at_fname);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
-	}
-
 	/*
 	 * Todo: use SMB_VFS_STATX() once it's available.
 	 */
@@ -1097,42 +1170,31 @@ static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
 	/* Might be a symlink. */
 	ret = SMB_VFS_LSTAT(conn, smb_dname);
 	if (ret != 0) {
-		TALLOC_FREE(parent_fname);
 		return map_nt_error_from_unix(errno);
 	}
 
-	if (S_ISLNK(smb_dname->st.st_ex_mode)) {
+	if (S_ISLNK(st.st_ex_mode)) {
 		/* Is what it points to a directory ? */
 		ret = SMB_VFS_STAT(conn, smb_dname);
 		if (ret != 0) {
-			TALLOC_FREE(parent_fname);
 			return map_nt_error_from_unix(errno);
 		}
-		if (!(S_ISDIR(smb_dname->st.st_ex_mode))) {
-			TALLOC_FREE(parent_fname);
+		if (!(S_ISDIR(st.st_ex_mode))) {
 			return NT_STATUS_NOT_A_DIRECTORY;
 		}
 	} else {
 		unlink_flags = AT_REMOVEDIR;
 	}
 
-	ret = SMB_VFS_UNLINKAT(conn,
-			       parent_fname->fsp,
-			       at_fname,
-			       unlink_flags);
+	ret = SMB_VFS_UNLINKAT(conn, dirfsp, at_fname, unlink_flags);
 	if (ret == 0) {
-		TALLOC_FREE(parent_fname);
-		notify_fname(conn, NOTIFY_ACTION_REMOVED,
-			     FILE_NOTIFY_CHANGE_DIR_NAME,
-			     smb_dname->base_name);
 		return NT_STATUS_OK;
 	}
 
 	if (!((errno == ENOTEMPTY) || (errno == EEXIST))) {
-		DEBUG(3,("rmdir_internals: couldn't remove directory %s : "
-			 "%s\n", smb_fname_str_dbg(smb_dname),
-			 strerror(errno)));
-		TALLOC_FREE(parent_fname);
+		DBG_NOTICE("couldn't remove directory %s : %s\n",
+			   smb_fname_str_dbg(smb_dname),
+			   strerror(errno));
 		return map_nt_error_from_unix(errno);
 	}
 
@@ -1149,16 +1211,11 @@ static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
 	}
 
 	/*
-	 * Check to see if the only thing in this directory are
+	 * Check to see if the only things in this directory are
 	 * files non-visible to the client. If not, fail the delete.
 	 */
 
-	status = OpenDir(talloc_tos(),
-			 conn,
-			 smb_dname,
-			 NULL,
-			 0,
-			 &dir_hnd);
+	status = OpenDir_from_pathref(talloc_tos(), fsp, NULL, 0, &dir_hnd);
 	if (!NT_STATUS_IS_OK(status)) {
 		/*
 		 * Note, we deliberately squash the error here
@@ -1169,289 +1226,27 @@ static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
 		goto err;
 	}
 
-	dirfsp = dir_hnd_fetch_fsp(dir_hnd);
+	status = can_delete_directory_hnd(dir_hnd);
+	TALLOC_FREE(dir_hnd);
 
-	while ((dname = ReadDirName(
-			dir_hnd, &dirpos, NULL, &talloced)) != NULL) {
-		struct smb_filename *smb_dname_full = NULL;
-		struct smb_filename *direntry_fname = NULL;
-		char *fullname = NULL;
-		int retval;
-
-		if (ISDOT(dname) || ISDOTDOT(dname)) {
-			TALLOC_FREE(talloced);
-			continue;
-		}
-		if (IS_VETO_PATH(conn, dname)) {
-			TALLOC_FREE(talloced);
-			continue;
-		}
-
-		fullname = talloc_asprintf(talloc_tos(),
-					   "%s/%s",
-					   smb_dname->base_name,
-					   dname);
-
-		if (fullname == NULL) {
-			TALLOC_FREE(talloced);
-			status = NT_STATUS_NO_MEMORY;
-			goto err;
-		}
-
-		smb_dname_full = synthetic_smb_fname(talloc_tos(),
-						     fullname,
-						     NULL,
-						     NULL,
-						     smb_dname->twrp,
-						     smb_dname->flags);
-		if (smb_dname_full == NULL) {
-			TALLOC_FREE(talloced);
-			TALLOC_FREE(fullname);
-			status = NT_STATUS_NO_MEMORY;
-			goto err;
-		}
-
-		retval = SMB_VFS_LSTAT(conn, smb_dname_full);
-		if (retval != 0) {
-			status = map_nt_error_from_unix(errno);
-			TALLOC_FREE(talloced);
-			TALLOC_FREE(fullname);
-			TALLOC_FREE(smb_dname_full);
-			goto err;
-		}
-
-		if (S_ISLNK(smb_dname_full->st.st_ex_mode)) {
-			/* Could it be an msdfs link ? */
-			if (lp_host_msdfs() &&
-			    lp_msdfs_root(SNUM(conn))) {
-				struct smb_filename *smb_atname;
-				smb_atname = synthetic_smb_fname(talloc_tos(),
-							dname,
-							NULL,
-							&smb_dname_full->st,
-							fsp->fsp_name->twrp,
-							fsp->fsp_name->flags);
-				if (smb_atname == NULL) {
-					TALLOC_FREE(talloced);
-					TALLOC_FREE(fullname);
-					TALLOC_FREE(smb_dname_full);
-					status = NT_STATUS_NO_MEMORY;
-					goto err;
-				}
-				if (is_msdfs_link(fsp, smb_atname)) {
-					TALLOC_FREE(talloced);
-					TALLOC_FREE(fullname);
-					TALLOC_FREE(smb_dname_full);
-					TALLOC_FREE(smb_atname);
-					DBG_DEBUG("got msdfs link name %s "
-						"- can't delete directory %s\n",
-						dname,
-						fsp_str_dbg(fsp));
-					status = NT_STATUS_DIRECTORY_NOT_EMPTY;
-					goto err;
-				}
-				TALLOC_FREE(smb_atname);
-			}
-
-			/* Not a DFS link - could it be a dangling symlink ? */
-			retval = SMB_VFS_STAT(conn, smb_dname_full);
-			if (retval == -1 && (errno == ENOENT || errno == ELOOP)) {
-				/*
-				 * Dangling symlink.
-				 * Allow delete as "delete veto files = yes"
-				 */
-				TALLOC_FREE(talloced);
-				TALLOC_FREE(fullname);
-				TALLOC_FREE(smb_dname_full);
-				continue;
-			}
-
-			DBG_DEBUG("got symlink name %s - "
-				"can't delete directory %s\n",
-				dname,
-				fsp_str_dbg(fsp));
-			TALLOC_FREE(talloced);
-			TALLOC_FREE(fullname);
-			TALLOC_FREE(smb_dname_full);
-			status = NT_STATUS_DIRECTORY_NOT_EMPTY;
-			goto err;
-		}
-
-		/* Not a symlink, get a pathref. */
-		status = synthetic_pathref(talloc_tos(),
-					   dirfsp,
-					   dname,
-					   NULL,
-					   &smb_dname_full->st,
-					   smb_dname->twrp,
-					   smb_dname->flags,
-					   &direntry_fname);
-		if (!NT_STATUS_IS_OK(status)) {
-			TALLOC_FREE(talloced);
-			TALLOC_FREE(fullname);
-			TALLOC_FREE(smb_dname_full);
-			goto err;
-		}
-
-		if (!is_visible_fsp(direntry_fname->fsp)) {
-			TALLOC_FREE(talloced);
-			TALLOC_FREE(fullname);
-			TALLOC_FREE(smb_dname_full);
-			TALLOC_FREE(direntry_fname);
-			continue;
-		}
-
-		/*
-		 * We found a client visible name.
-		 * We cannot delete this directory.
-		 */
-		DBG_DEBUG("got name %s - "
-			"can't delete directory %s\n",
-			dname,
-			fsp_str_dbg(fsp));
-		TALLOC_FREE(talloced);
-		TALLOC_FREE(fullname);
-		TALLOC_FREE(smb_dname_full);
-		TALLOC_FREE(direntry_fname);
+	if (!NT_STATUS_IS_OK(status)) {
 		status = NT_STATUS_DIRECTORY_NOT_EMPTY;
 		goto err;
 	}
 
-	/* Do a recursive delete. */
-	RewindDir(dir_hnd,&dirpos);
-
-	while ((dname = ReadDirName(
-			dir_hnd, &dirpos, NULL, &talloced)) != NULL) {
-		struct smb_filename *direntry_fname = NULL;
-		struct smb_filename *smb_dname_full = NULL;
-		char *fullname = NULL;
-		bool do_break = true;
-		int retval;
-
-		if (ISDOT(dname) || ISDOTDOT(dname)) {
-			TALLOC_FREE(talloced);
-			continue;
-		}
-
-		fullname = talloc_asprintf(ctx,
-					   "%s/%s",
-					   smb_dname->base_name,
-					   dname);
-
-		if (fullname == NULL) {
-			status = NT_STATUS_NO_MEMORY;
-			goto err_break;
-		}
-
-		smb_dname_full = synthetic_smb_fname(talloc_tos(),
-						     fullname,
-						     NULL,
-						     NULL,
-						     smb_dname->twrp,
-						     smb_dname->flags);
-		if (smb_dname_full == NULL) {
-			status = NT_STATUS_NO_MEMORY;
-			goto err_break;
-		}
-
-		/*
-		 * Todo: use SMB_VFS_STATX() once that's available.
-		 */
-
-		retval = SMB_VFS_LSTAT(conn, smb_dname_full);
-		if (retval != 0) {
-			status = map_nt_error_from_unix(errno);
-			goto err_break;
-		}
-
-		/*
-		 * We are only dealing with VETO'ed objects
-		 * here. If it's a symlink, just delete the
-		 * link without caring what it is pointing
-		 * to.
-		 */
-		if (S_ISLNK(smb_dname_full->st.st_ex_mode)) {
-			direntry_fname = synthetic_smb_fname(talloc_tos(),
-							dname,
-							NULL,
-							&smb_dname_full->st,
-							smb_dname->twrp,
-							smb_dname->flags);
-			if (direntry_fname == NULL) {
-				status = NT_STATUS_NO_MEMORY;
-				goto err_break;
-			}
-		} else {
-			status = synthetic_pathref(talloc_tos(),
-						   dirfsp,
-						   dname,
-						   NULL,
-						   &smb_dname_full->st,
-						   smb_dname->twrp,
-						   smb_dname->flags,
-						   &direntry_fname);
-			if (!NT_STATUS_IS_OK(status)) {
-				goto err_break;
-			}
-
-			if (!is_visible_fsp(direntry_fname->fsp)) {
-				TALLOC_FREE(fullname);
-				TALLOC_FREE(smb_dname_full);
-				TALLOC_FREE(talloced);
-				TALLOC_FREE(direntry_fname);
-				continue;
-			}
-		}
-
-		unlink_flags = 0;
-
-		if (smb_dname_full->st.st_ex_mode & S_IFDIR) {
-			status = recursive_rmdir(ctx, conn, smb_dname_full);
-			if (!NT_STATUS_IS_OK(status)) {
-				goto err_break;
-			}
-			unlink_flags = AT_REMOVEDIR;
-		}
-
-		retval = SMB_VFS_UNLINKAT(conn,
-					  dirfsp,
-					  direntry_fname,
-					  unlink_flags);
-		if (retval != 0) {
-			status = map_nt_error_from_unix(errno);
-			goto err_break;
-		}
-
-		/* Successful iteration. */
-		do_break = false;
-
-	err_break:
-		TALLOC_FREE(fullname);
-		TALLOC_FREE(smb_dname_full);
-		TALLOC_FREE(talloced);
-		TALLOC_FREE(direntry_fname);
-		if (do_break) {
-			break;
-		}
+	status = recursive_rmdir_fsp(fsp);
+	if (!NT_STATUS_IS_OK(status)) {
+		status = NT_STATUS_DIRECTORY_NOT_EMPTY;
+		goto err;
 	}
 
-	/* If we get here, we know NT_STATUS_IS_OK(status) */
-	SMB_ASSERT(NT_STATUS_IS_OK(status));
-
 	/* Retry the rmdir */
-	ret = SMB_VFS_UNLINKAT(conn,
-			       parent_fname->fsp,
-			       at_fname,
-			       AT_REMOVEDIR);
+	ret = SMB_VFS_UNLINKAT(conn, dirfsp, at_fname, AT_REMOVEDIR);
 	if (ret != 0) {
 		status = map_nt_error_from_unix(errno);
 	}
 
   err:
-
-	TALLOC_FREE(dir_hnd);
-	TALLOC_FREE(parent_fname);
-
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_NOTICE("couldn't remove directory %s : "
 			 "%s\n", smb_fname_str_dbg(smb_dname),
@@ -1459,23 +1254,21 @@ static NTSTATUS rmdir_internals(TALLOC_CTX *ctx, struct files_struct *fsp)
 		return status;
 	}
 
-	notify_fname(conn, NOTIFY_ACTION_REMOVED,
-		     FILE_NOTIFY_CHANGE_DIR_NAME,
-		     smb_dname->base_name);
-
 	return status;
 }
 
 /****************************************************************************
- Close a directory opened by an NT SMB call. 
+ Close a directory opened by an NT SMB call.
 ****************************************************************************/
-  
+
 static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 				enum file_close_type close_type)
 {
 	connection_struct *conn = fsp->conn;
 	struct close_share_mode_lock_state lck_state = {};
 	bool changed_user = false;
+	struct smb_filename *parent_fname = NULL;
+	struct smb_filename *base_fname = NULL;
 	NTSTATUS status = NT_STATUS_OK;
 	NTSTATUS status1 = NT_STATUS_OK;
 	NTSTATUS notify_status;
@@ -1483,7 +1276,7 @@ static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 
 	SMB_ASSERT(fsp->fsp_flags.is_fsa);
 
-	if (fsp->conn->sconn->using_smb2) {
+	if (conn_using_smb2(fsp->conn->sconn)) {
 		notify_status = NT_STATUS_NOTIFY_CLEANUP;
 	} else {
 		notify_status = NT_STATUS_OK;
@@ -1509,14 +1302,14 @@ static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("share_mode_entry_prepare_lock_del() failed for %s - %s\n",
 			fsp_str_dbg(fsp), nt_errstr(status));
-		return status;
+		log_stack_trace();
+		goto close_fd;
 	}
 
-	/*
-	 * We don't have directory leases yet, so assert it in order
-	 * to skip release_file_oplock().
-	 */
-	SMB_ASSERT(fsp->oplock_type == NO_OPLOCK);
+	/* Remove the oplock before potentially deleting the file. */
+	if (fsp->oplock_type != NO_OPLOCK) {
+		release_file_oplock(fsp);
+	}
 
 	/*
 	 * NT can set delete_on_close of the last open
@@ -1555,10 +1348,22 @@ static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 		changed_user = true;
 	}
 
+	status = parent_pathref(talloc_tos(),
+				conn->cwd_fsp,
+				fsp->fsp_name,
+				&parent_fname,
+				&base_fname);
+	if (!NT_STATUS_IS_OK(status)) {
+		DBG_DEBUG("parent_pathref(%s) failed: %s\n",
+			  fsp_str_dbg(fsp), nt_errstr(status));
+		goto done;
+	}
 	if ((fsp->conn->fs_capabilities & FILE_NAMED_STREAMS)
 	    && !is_ntfs_stream_smb_fname(fsp->fsp_name)) {
 
-		status = delete_all_streams(fsp->conn, fsp->fsp_name);
+		status = delete_all_streams(fsp,
+					    parent_fname->fsp,
+					    base_fname);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(5, ("delete_all_streams failed: %s\n",
 				  nt_errstr(status)));
@@ -1566,11 +1371,14 @@ static NTSTATUS close_directory(struct smb_request *req, files_struct *fsp,
 		}
 	}
 
-	status = rmdir_internals(talloc_tos(), fsp);
+	status = rmdir_internals(fsp, parent_fname->fsp, base_fname);
 
-	DEBUG(5,("close_directory: %s. Delete on close was set - "
+	TALLOC_FREE(parent_fname);
+
+	DBG_INFO("%s. Delete on close was set - "
 		 "deleting directory returned %s.\n",
-		 fsp_str_dbg(fsp), nt_errstr(status)));
+		 fsp_str_dbg(fsp),
+		 nt_errstr(status));
 
 	/*
 	 * Ensure we remove any change notify requests that would
@@ -1598,6 +1406,31 @@ done:
 
 	remove_pending_change_notify_requests_by_fid(fsp, notify_status);
 
+	if (lck_state.delete_object && NT_STATUS_IS_OK(status)) {
+		const struct smb2_lease *lease = fsp_get_smb2_lease(fsp);
+
+		if (lease != NULL) {
+			/*
+			 * If parent lease key of handle on which delete
+			 * disposition was set does not match the parent lease
+			 * key of last closed handle, break all leases on the
+			 * parent directory.
+			 */
+			if (!smb2_lease_key_equal(&lease->parent_lease_key,
+						  &lck_state.parent_lease_key))
+			{
+				lease = NULL;
+			}
+		}
+		notify_fname(conn,
+			     NOTIFY_ACTION_REMOVED |
+			     NOTIFY_ACTION_DIRLEASE_BREAK,
+			     FILE_NOTIFY_CHANGE_DIR_NAME,
+			     fsp->fsp_name,
+			     lease);
+	}
+
+close_fd:
 	status1 = fd_close(fsp);
 
 	if (!NT_STATUS_IS_OK(status1)) {
@@ -1615,7 +1448,7 @@ done:
 /****************************************************************************
  Rundown all SMB-related dependencies of a files struct
 ****************************************************************************/
-  
+
 NTSTATUS close_file_smb(struct smb_request *req,
 			struct files_struct *fsp,
 			enum file_close_type close_type)
@@ -1717,26 +1550,24 @@ void msg_close_file(struct messaging_context *msg_ctx,
 			struct server_id server_id,
 			DATA_BLOB *data)
 {
+	struct oplock_break_message msg;
+	enum ndr_err_code ndr_err;
 	files_struct *fsp = NULL;
-	struct file_id id;
-	struct share_mode_entry e;
 	struct smbd_server_connection *sconn =
 		talloc_get_type_abort(private_data,
 		struct smbd_server_connection);
 
-	message_to_share_mode_entry(&id, &e, (char *)data->data);
-
-	if(DEBUGLVL(10)) {
-		char *sm_str = share_mode_str(NULL, 0, &id, &e);
-		if (!sm_str) {
-			smb_panic("talloc failed");
-		}
-		DEBUG(10,("msg_close_file: got request to close share mode "
-			"entry %s\n", sm_str));
-		TALLOC_FREE(sm_str);
+	ndr_err = ndr_pull_struct_blob_all_noalloc(
+		data,
+		&msg,
+		(ndr_pull_flags_fn_t)ndr_pull_oplock_break_message);
+	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
+		DBG_DEBUG("ndr_pull_oplock_break_message failed: %s\n",
+			  ndr_errstr(ndr_err));
+		return;
 	}
 
-	fsp = file_find_dif(sconn, id, e.share_file_id);
+	fsp = file_find_dif(sconn, msg.id, msg.share_file_id);
 	if (!fsp) {
 		DEBUG(10,("msg_close_file: failed to find file.\n"));
 		return;
